@@ -35,7 +35,26 @@
  *
  */
 
+/**********************************************************
+*
+*   S E T U P
+*
+***********************************************************/
+
+$host = "geekette.localdomain"; // set serve'rs host here
+$printer_uri = "/printers/epson"; // set printer uri here
+$language = "fr_fr";
+$user = "test"; // valid user from lpadmin group
+$password = "test"; // his password
+$document_uri = "http://localhost/";
+
 error_reporting(E_ALL);
+/******************
+*
+* END OF SETUP
+*
+*******************/
+
 
 function __autoload($class_name) {
    require_once $class_name . '.php';
@@ -43,55 +62,59 @@ function __autoload($class_name) {
    
 //require_once("CupsPrintIPP.php"); // other files are loaded by it
 
-echo "<h1>PHP PrintIPP example</h1>";
+echo "<html><head></head><body><h1>PHP PrintIPP example</h1>";
 
 echo "Note that all options are not showed here<br />";
 
 //$ipp = new PrintIPP(); // Basic PrintIPP
 //$ipp = new ExtendedPrintIPP(); // extended IPP
 $ipp = new CupsPrintIPP(); // extended IPP with CUPS specific stuff
-//$ipp = new PrintIPP(); // Basic IPP (print job, print uri, get printer's attributes, cancel jobs)
 
+$ipp->debug_level = 3; // Debugging very verbose
 
-
-//$ipp->debug_level = 3; // Debugging very verbose
+$ipp->setLog('/tmp/printipp','file',1); // logging almost quiet
 $ipp->setLog('/tmp/printipp','file',2); // logging verbose
-$ipp->setHost("localhost");//Put your printer IP or hostname/fqdn here
+$ipp->setLog('/tmp/printipp','file',3); // logging very verbose
 
+$ipp->setHost("localhost");
 //$ipp->setHost("toto");// Resistance test
-$ipp->setHost("geekette");// Resistance test
-
+$ipp->setHost($host); //Put your printer IP or hostname/fqdn here
 
+$ipp->setPrinterURI($printer_uri); // Set printer URI here
 //$ipp->setPrinterURI("ipp://localhost:631/printers/epson"); // Set printer URI here
-$ipp->setPrinterURI("/printers/epson"); // Set printer URI here
 //$ipp->setPrinterURI("/printers/foo"); // => abort "client-error-not-found"
 
 $ipp->setUserName("php IPP tester");
+
 $ipp->setFidelity(); // printing abort if every attribute could not be set on printer. NOTE: CUPS do not abort :)
 
-
-//$ipp->setMimeMediaType('text/html');
-//$ipp->setMimeMediaType('foo/bar'); // => client-error-document-format-not-supported
-//$ipp->setMimeMediaType('application/postscript');
 $ipp->setCharset('utf-8');
-$ipp->setLanguage('fr_fr');
+$ipp->setLanguage($language);
 
-$ipp->setAuthentification("test","test"); // username & password 
+$ipp->setAuthentification($user,$password); // username & password 
 
+$j = 0;
+$test = 1;
 /* getting printer's attributes */
-/*
-$ipp->getPrinterAttributes();
+echo "<br /><br /><br />TEST ".$test++."<br />";
+echo "OPERATION  ".$j++."<br />";
+echo "Getting Printer's attributes: ". $ipp->getPrinterAttributes() . "<br />";
+printf('$ipp->status[0] = %s <br /><br />', $ipp->status[0]);
 echo "Printer's attributes :<pre>\n"; print_r($ipp->printer_attributes); echo "</pre>";
 
 /* getting printers (CUPS extention) */
-/*
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ".$j++."<br />";
 echo "Available printers [CUPS EXTENTION]:<pre>" ; 
 echo $ipp->getPrinters() . "\n";
 print_r($ipp->available_printers);
 echo "</pre>";
-/*
+
+
 for ($i = 0 ; $i < count($ipp->available_printers) ; $i ++) {
-//$ipp->setPrinterURI($ipp->available_printers[$i]); // Set printer URI here
+$ipp->setPrinterURI($ipp->available_printers[$i]);
+echo "OPERATION ".$j ++."<br />";
 $ipp->getPrinterAttributes();
 echo "Printer attributes for printer $i:<pre>\n"; print_r($ipp->printer_attributes); echo "</pre>";
 if (isset($ipp->printer_attributes->printer_type->_value2)
@@ -106,73 +129,106 @@ if (isset($ipp->printer_attributes->printer_type->_value3)
     echo "Document formats supported:<br /><pre>";
 
     $pointer = "_value0";
-    for ($j = 0 ; isset($ipp->printer_attributes->document_format_supported->$pointer); $j++) {
+    for ($k = 0 ; isset($ipp->printer_attributes->document_format_supported->$pointer); $k++) {
         echo $ipp->printer_attributes->document_format_supported->$pointer . "\n";
-        $pointer = "_value" . ($j + 1);
+        $pointer = "_value" . ($k + 1);
         }
     echo "</pre>";
 
 echo "------- END FOR PRINTER $i -------------<br /n>";
 }
 
+
+/* resetting printer uri to those configured */
+$ipp->setPrinterURI($printer_uri);
+
+
 /* printing a string */
-/*
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
 $ipp->setJobName("PHP Test: Text String",true); // default is false: number is automagically appended
-$ipp->setRawText();
-$ipp->setData("This is a RAW string");//string to print.
-//echo "Print RAW String: ".$ipp->printJob()."<br />";
-echo "Job Attributes:<pre>\n" ; print_r($ipp->job_attributes) ; echo "\n</pre>\n";
-$ipp->unsetRawText();
 $ipp->setData("This is a text string");
-//echo "Print String: ".$ipp->printJob()."</br />";
+echo "Print String: ".$ipp->printJob()."</br />";
+echo "Job Attributes:<pre>\n" ; print_r($ipp->job_attributes) ; echo "\n</pre>\n";
+
+// for restart job operation
+$first_job = $ipp->last_job;
 
 /* printing a text file */
-/*
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
         // HINT: you _must_ supply a charset or set output as raw text
         // note that mimeMediaType is resetted to octet-stream after each call of printJob (this is a feature).
-//$ipp->setRawText();
+echo "OPERATION ". $j ++ ."<br />";
 $ipp->setCharset('us-ascii');
 $ipp->setMimeMediaType('text/plain');
+$ipp->setJobName("PHP Test: US ASCII file",true); // default is false: number is automagically appended
 
 $ipp->setData("./testfiles/test.txt");//Path to file.
-echo "Job status: ".$ipp->printJob("epson")."\n";
+echo "US ASCII file Job status: ".$ipp->printJob("epson")."<br />";
 
 
 /* printing a document by URI */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+echo "Job Print URI status: ".$ipp->printUri($document_uri)."<br />";
+
 
-echo "Job Print URI status: ".$ipp->printUri("http://localhost/")."<br />";
 
 /* printing a text file in utf-8 */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
 
-
-
+echo "OPERATION ". $j ++ ."<br />";
 $ipp->setUserName("foo bar");
-$ipp->setAuthentification("test","test"); // username & password 
-$ipp->setDocumentName("testfile with UTF-8 characters");
 $ipp->setCharset('utf-8');
-// attributes without dedicated function
 $ipp->setMimeMediaType('text/plain'); // if autodetection do not work
+// attributes without dedicated function
 $ipp->setAttribute('orientation-requested','landscape');
 $ipp->setAttribute('number-up',2);
-$ipp->setAttribute('compression','gzip');
-//$ipp->setData("./testfiles/test-utf8-compressed.txt.gz");//Path to file.
+$ipp->setDocumentName("testfile with UTF-8 characters, gzipped");
+$ipp->setJobName("testfile with UTF-8 characters, gzipped");
+$ipp->setDocumentName("testfile with UTF-8 characters");
 $ipp->setData("./testfiles/test-utf8.txt");//Path to file
-//$ipp->setData("test fin");
-//echo "Purge jobs: ".$ipp->purgeJobs()."<br />";
-//sleep (1);
-//sleep(1);
-//echo "Printing Job: ".$ipp->printJob()."<br />";
-echo "Printing Job: ".$ipp->printJob()."<br />";
-$ipp->debug_level = 3;
+echo "Printing testfile with UTF-8 characters: ".$ipp->printJob()."<br />";
 
-//$ipp->getJobs($my_jobs=true,$limit=0,"completed",$subset=true); // give only a usefull subset of jobs's attributes
-echo "Getting Jobs: ".$ipp->getJobs(true,0,"",true)."<br />"; // defaults to $my_jobs=true,$limit=0 (no limit),$which_jobs=not-completed,$subset=false
+/* printing a gzipped file */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+$ipp->setAttribute('compression','gzip');
+$ipp->setData("./testfiles/test-utf8-compressed.txt.gz");//Path to file.
+echo "Printing testfile with UTF-8 characters, gzipped: ".$ipp->printJob()."<br />";
+$ipp->unsetAttribute('compression');
+
+
+/* getting not-completed jobs */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+echo "Getting NOT PROCESSED Jobs: ".$ipp->getJobs(true,0,"")."<br />"; // defaults to $my_jobs=true,$limit=0 (no limit),$which_jobs=not-completed,$subset=false 
 
 echo "Job 0 state: ".$ipp->jobs_attributes->job_0->job_state->_value0."<br />";
 echo "Job 0 state-reasons: ".$ipp->jobs_attributes->job_0->job_state_reasons->_value0."<br />";
 echo "<pre>";print_r($ipp->jobs_attributes); echo "</pre>";
 
-/*$ipp->getJobAttributes($ipp->last_job,false,'job-template');
+/* getting all jobs (subset of attributes) */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+echo "Getting ALL Jobs: ".$ipp->getJobs(true,0,"completed",true)."<br />";
+
+echo "Job 0 state: ".$ipp->jobs_attributes->job_0->job_state->_value0."<br />";
+echo "Job 0 state-reasons: ".$ipp->jobs_attributes->job_0->job_state_reasons->_value0."<br />";
+echo "<pre>";print_r($ipp->jobs_attributes); echo "</pre>";
+
+
+/* getting a job's attributes */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+echo "Getting last job's attributes: ".$ipp->getJobAttributes($ipp->last_job,false,'job-template')."<br />";
                     
 $job_state = $ipp->job_attributes->job_state->_value0;
 echo "Job-State: $job_state<br />";
@@ -182,94 +238,100 @@ echo "Job-State-Reasons: $job_state_reasons<br />";
  
 echo "<pre>";print_r($ipp->job_attributes); echo "</pre>";
 
-//$ipp->setData("This is a test");
-/*
+
+
+/* printing selected pages from a document */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
 $ipp->setAttribute('media','A7');
-$ipp->setData("./testfiles/documentation/COPYING");
+$ipp->setAttribute('number-up',4);
+$ipp->setData("./testfiles/COPYING");
 $ipp->setPageRanges('1:2 5:6');
-$success = $ipp->printJob();
-
+echo "Printing selected pages from document by 4 pages on a single A7 media: ". $ipp->printJob() ."<br />";
 $ipp->setPageRanges('');
-$ipp->setData("This is a test for strings");
-
-/*
-$success = $ipp->printJob();
-if (!$success) 
-    echo "<br />PrintJob status: OPERATION FAILED<br />";
-else
-    echo "PrintJob status ".$success."<br />";
 
+$ipp->unsetAttribute('media');
+$ipp->unsetAttribute('number-up');
 
 /* printing a postcript file */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
 
-//$ipp->setAuthentification("test","test"); // username & password 
-//$ipp->setData("./testfiles/test.ps");
-//echo "Job status: ".$ipp->printJob("epson")."<br />";
+echo "OPERATION ". $j ++ ."<br />";
+$ipp->setMimeMediaType(); // => autodetection
+$ipp->setData("./testfiles/test.ps");
+echo "Printing Postscript Job status: ".$ipp->printJob("epson")."<br />";
 
 
 /* printing a png  file */
-//$ipp->setData("./testfiles/test.png");
-//echo "Job status: ".$ipp->printJob("epson")."<br />";
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+$ipp->setData("./testfiles/test.png");
+echo "Printing png Job status: ".$ipp->printJob("epson")."<br />";
 
 /* printing and cancelling job */
-/*
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
 echo "Printing and Cancelling a job<br />";
-//$ipp->setData("./testfiles/test.ps");
-//$ipp->setData("./testfiles/test-compressed.html.gz");
-//$ipp->setData("./testfiles/test.html");
+$ipp->setData("./testfiles/test.ps");
+echo "OPERATION ". $j ++ ."<br />";
 echo "Job status: ".$ipp->printJob()."<br />";
-*/
-/*
 $job = $ipp->last_job;
-$ipp->setAuthentification("test","test"); // username & password 
 $ipp->setMessage(sprintf(_("job %s cancelled"),$job));
+echo "OPERATION ". $j ++ ."<br />";
 echo "Cancel status: ".$ipp->cancelJob($job)."<br />";
 
 /* printing strings, no form feed */
-/*
-$ipp->setAuthentification("test","test"); // username & password 
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
 $ipp->setRawText();
 $ipp->unsetFormFeed();
 
+echo "Printing RAW TEXT strings<br />";
+
 $ipp->setData("This is a line\n");
+echo "OPERATION ". $j ++ ."<br />";
 echo "Job status: ".$ipp->printJob("epson")."<br />";
 
 $ipp->setData("This is half a line ");
+echo "OPERATION ". $j ++ ."<br />";
 echo "Job status: ".$ipp->printJob("epson")."<br />";
 
 $ipp->setData("This is a end of line\n");
+echo "OPERATION ". $j ++ ."<br />";
 echo "Job status: ".$ipp->printJob("epson")."<br />";
 
 // set copies to 2 (same sheet of paper: form feed is unset)
 $ipp->setData("This lines must appeared twice\r\n");
 $ipp->setCopies(2);
+echo "OPERATION ". $j ++ ."<br />";
 echo "Job status: ".$ipp->printJob("epson")."<br />";
 $ipp->setCopies(1);
 
 // printing string, then form feed
+echo "OPERATION ". $j ++ ."<br />";
 $ipp->setFormFeed();
 $ipp->setData("End of test");
 echo "Job status: ".$ipp->printJob("epson")."<br />";
 
 $ipp->unsetRawText();
+
+
+/* printing a file to see if unsetRawText works */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
 $ipp->setData("./testfiles/test.ps");
-//$ipp->setData("./testfiles/test-compressed.html.gz");
-//$ipp->setData("./testfiles/test.html");
-echo "Job status: ".$ipp->printJob()."<br />";
+echo "Job status (text file after strings): ".$ipp->printJob()."<br />";
 
-/* send a buggy request */
-/*
-$ipp->setData("This is an error : nothing printed\n");
-$ipp->generateError ("request_body_malformed");
-echo "Job status: ".$ipp->printJob("epson");
 
-$ipp->resetError("request_body_malformed");
-*/
-/*
-$ipp->debug_level = 3;
-$ipp->setAuthentification("test","test"); // username & password 
+/* multiple document handling */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
 $ipp->setUserName("test");
 $ipp->setCopies(1);
+echo "OPERATION ". $j ++ ."<br />";
 echo "Create-Job: ".$ipp->createJob(). "<br />";
 printf("Job is: %s<br />",$job = $ipp->last_job);
 
@@ -277,19 +339,28 @@ echo "<pre>";print_r($ipp->job_attributes);echo "</pre>\n";
 
 $ipp->setDocumentName("test-utf8.txt");
 $ipp->setData("./testfiles/test-utf8.txt");
+echo "OPERATION ". $j ++ ."<br />";
 echo "Sending document: " . $ipp->sendDocument($job) . "<br />\n";
 
+echo "OPERATION ". $j ++ ."<br />";
+echo "Sending URI: ".$ipp->sendURI('http://localhost',$job). "<br />\n";
+
 $ipp->setDocumentName("text string");
 $ipp->setData("This is the string of second document");
-echo "Sending document: " . $ipp->sendDocument($job,$last=true) . "<br />\n";
+echo "OPERATION ". $j ++ ."<br />";
+echo "Sending text string as _last_ document: " . $ipp->sendDocument($job,$last=true) . "<br />\n";
 
 // must be refused. Hem: CUPS is very smart, it accepts :)
-echo "Sending document: " . $ipp->sendDocument($job,$last=true) . "<br />\n";
+echo "OPERATION ". $j ++ ."<br />";
+echo "Sending document (must be refused): " . $ipp->sendDocument($job,$last=true) . "<br />\n";
 
 
+/* try to validate a job with filetype printer server can't handle */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
 
 $ipp->setMimeMediaType("application/x-foobar");
 
+echo "OPERATION ". $j ++ ."<br />";
 echo "Validate-Job: ".$ipp->validateJob()."<br />";
 
 foreach ($ipp->job_attributes as $name => $job_attribute) {
@@ -298,21 +369,98 @@ foreach ($ipp->job_attributes as $name => $job_attribute) {
 }
 reset($ipp->job_attributes);
 echo "Details:<pre>"; print_r($ipp->job_attributes) ; echo "</pre>";
+
+
+/* Printing then holding a job */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+echo "Printing a document :".$ipp->printJob()."<br />";
+echo "Job is: ".$job = $ipp->last_job."<br />";
+//sleep(1);
+echo "OPERATION ". $j ++ ."<br />";
+echo "Holding the job for an indefinite period: ".$ipp->holdJob($job,'indefinite')."<br />";
+echo "OPERATION ". $j ++ ."<br />";
+echo "Getting job state : ".$ipp->getJobAttributes($job)."<br />";
+echo "Job State: ".$ipp->job_attributes->job_state->_value0."<br />";
+echo "Job State Reason: ".$ipp->job_attributes->job_state_reasons->_value0."<br />";
+
+/* releasing the job */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+echo "Releasing the job for: ".$ipp->releaseJob($job,'indefinite')."<br />";
+echo "OPERATION ". $j ++ ."<br />";
+echo "Getting job state : ".$ipp->getJobAttributes($job)."<br />";
+echo "Job State: ".$ipp->job_attributes->job_state->_value0."<br />";
+echo "Job State Reason: ".$ipp->job_attributes->job_state_reasons->_value0."<br />";
+
+
+/* restarting a job */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "OPERATION ". $j ++ ."<br />";
+echo "Restarting the job $first_job (if completed!): ".$ipp->restartJob($first_job)."<br />";
+echo "OPERATION ". $j ++ ."<br />";
+echo "Getting job state : ".$ipp->getJobAttributes($first_job)."<br />";
+echo "Job State: ".$ipp->job_attributes->job_state->_value0."<br />";
+echo "Job State Reason: ".$ipp->job_attributes->job_state_reasons->_value0."<br />";
+
+
 /* purging jobs for a printer */
-/*
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
 $ipp->setUserName("test");
-$ipp->setAuthentification("test","test"); // username & password 
 $ipp->setPrinterURI("ipp://localhost:631/printers/epson"); // Set printer URI here
-//echo "Purge-Jobs: ". $ipp->purgeJobs() ."<br />";
+echo "Purge-Jobs for printer $printer_uri: ". $ipp->purgeJobs() ."<br />";
 
 /* purging jobs for all printers */
-/*
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
 $ipp->setPrinterURI("ipp://localhost:631/printers/"); // => all printers
-echo "Purge-Jobs: ". $ipp->purgeJobs() ."<br />";
+echo "Purge-Jobs (all printers): ". $ipp->purgeJobs() ."<br />";
 
 
+/* pausing printer */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+$ipp->setPrinterURI($printer_uri);
+echo "Pausing Printer $printer_uri, then sleep 1 second: ".$ipp->pausePrinter()."<br />";
+sleep(1);
+echo "OPERATION ". $j ++ ."<br />";
+echo "Getting printer's attributes: ".$ipp->getPrinterAttributes()."<br />";
+echo "Printer State: ".$ipp->printer_attributes->printer_state->_value0."<br />";
+
+/* Resuming printer */
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+
+echo "OPERATION ". $j ++ ."<br />";
+echo "Resuming Printer $printer_uri, then sleep 1 second: ".$ipp->resumePrinter()."<br />";
+sleep(1);
+echo "OPERATION ". $j ++ ."<br />";
+echo "Getting printer's attributes: ".$ipp->getPrinterAttributes()."<br />";
+echo "Printer State: ".$ipp->printer_attributes->printer_state->_value0."<br />";
+
+
+/* send a buggy request */
+
+echo "<br /><br /><br /> TEST ".$test ++."<br />";
+echo "OPERATION ". $j ++ ."<br />";
+$ipp->setData("This is an error : nothing printed\n");
+$ipp->generateError ("request_body_malformed");
+echo "Sending a buggy request. status: ".$ipp->printJob()."<br />";
+
+$ipp->resetError("request_body_malformed");
+
+
+/**/
+
+echo "END OF OPERATIONS <br /><br /><br />";
 
 /* get informations about jobs and status */
+
 echo "Available printers:<br />\n" ;
 echo "<pre>\n";
 print_r($ipp->available_printers);
@@ -340,7 +488,13 @@ echo "</pre>";
 
 
 /* get debugging informations */
+
 echo "<h3>Debug</h3><pre>";
 $ipp->printDebug();
 echo "</pre>";
+
+
+/* end of test */
+
+echo "END OF TESTFILE</body></html>";
 ?>
